/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

using System;
using System.Collections;
using System.Text;
using Seapine.SurroundSCM.API;

namespace SCMMigrate
{
    class scmEngine
    {
        public SSCMContext context;
        string scmUser;
        string scmPassword;
        string scmServer;
        uint scmPort;

        string oldMainline;
        string oldBranch;
        string newMainline;

        string workDir = "C:\\migrate";

        public scmEngine(String pscmUser, String pscmPassword, String pscmServer, String pscmPort)
        {

            scmUser = pscmUser;
            scmPassword = pscmPassword;
            scmServer = pscmServer;
            scmPort = Convert.ToUInt16(pscmPort);
        }
        public void Execute(String curBranch,String curMainline ,String pnewMainline)
        {
            oldMainline = curMainline;
            oldBranch = curBranch;
            newMainline = pnewMainline;
            
            if (login())
            {
                
                //We need to get the root of the branch.
                string rootRepo = "";
                SetContext(oldBranch, oldMainline);
                Logger.WriteLine("Retrieving root repository of mainline");
                SSCMResult result = SSCMAPI.ListRootRepo(context, out rootRepo);
                if (result.IsOk())
                {
                    CheckChildRepos(rootRepo);
                }
                else
                {
                    Logger.WriteLine("Unable to get root repository from branch.\n");
                    Logger.WriteLine("Error reported: \n");
                    Logger.WriteLine(SSCMAPI.GetLastError(result));

                }
                
            }


        }
        
        private void SetContext(String branch, String Mainline)
        {
            context.SetBranch(branch);
            context.SetMainline(Mainline);
        }
        private bool login()
        {
            bool isLoggedIn = false;
            Logger.WriteLine("Logging into Surround SCM");
            try
            {
                SSCMResult result = SSCMAPI.Connect(scmServer, scmPort, scmUser, scmPassword, out context);
                if (result.IsOk())
                {
                    isLoggedIn = true;
                }
                else
                {
                    Logger.WriteLine("Unable to log in.\n");
                    Logger.WriteLine("Error reported: \n");
                    Logger.WriteLine(SSCMAPI.GetLastError(result));

                }
            }
            catch (Exception fcc)
            {
                Logger.WriteLine("Unable to log in.\n");
                Logger.WriteLine("Error reported: \n");
                Logger.WriteLine(fcc.Message);

            }
            return isLoggedIn;
                

        }
        //Method takes the 'source' repo path and converts it to the
        //'destination' repo path
        private string ConvertRepoPath(string repoPath)
        {
            string rootRepo = "";
            //First we need to see if we are at the root of the source branch
            SetContext(oldBranch, oldMainline);
            SSCMResult result = SSCMAPI.ListRootRepo(context,out rootRepo);
            if (rootRepo == repoPath)
            {
                repoPath = newMainline;
            }
            else
            {
                repoPath = repoPath.Replace(rootRepo, newMainline);
            }
            
            return repoPath;
        }
        private void CheckChildRepos(string curRepo)
        {
            //First, let's get the equivalent of curRepo for the new mainline
            String newRepoPath = ConvertRepoPath(curRepo);
            //if we are not at the root, then we create new repos
            if (newRepoPath.Contains("/"))
            {
                String parentRepo = newRepoPath.Substring(0,newRepoPath.LastIndexOf("/"));
                String newRepo = newRepoPath.Substring(newRepoPath.LastIndexOf("/") + 1);
                SetContext(newMainline, newMainline);
                SSCMResult result = SSCMAPI.CreateRepo(context,parentRepo,newRepo,"repo added by API",0);
                if (!(result.IsOk()))
                {
                    Logger.WriteLine("Unable to create repository " + newRepo);
                    Logger.WriteLine("Error reported: \n");
                    Logger.WriteLine(SSCMAPI.GetLastError(result));

                }
                SetContext(oldBranch, oldMainline);
            }
            SSCMRepo[] repos = new SSCMRepo[0];
            CheckFilesInRepo(curRepo, newRepoPath);
            SSCMResult listResult = SSCMAPI.ListSubRepos(context, curRepo, out repos);
            if (listResult.IsOk())
            {
                for (int i = 0; i < repos.Length; i++)
                {
                    CheckChildRepos(repos[i].GetRepository());
                    
                }
            }
            else
            {
                Logger.WriteLine("Error getting child repositories from"+ curRepo +".\n");
                Logger.WriteLine("Error reported: \n");
                Logger.WriteLine(SSCMAPI.GetLastError(listResult));


            }

        }
        //Checks all files and historic events and copies them to new branch
        private void CheckFilesInRepo(string oldRepo, string newRepo)
        {
            SSCMFile[] scmfiles = new SSCMFile[0];
            SSCMResult fileResult = SSCMAPI.ListFilesInRepo(context, oldRepo, out scmfiles);
            if (fileResult.IsOk())
            {
                if (scmfiles.Length > 0)
                {
                    for (int i = 0; i < scmfiles.Length; i++)
                    {
                        FileHistoryItem[] historyItems = new FileHistoryItem[0];
                        Logger.WriteLine(scmfiles[i].GetName());
                        SSCMResult historyResult = SSCMAPI.FileHistory(context, oldRepo, scmfiles[i].GetName(), false, false, FileAction.AllActions, out historyItems);
                        if (historyResult.IsOk())
                        {
                            for (int j = 0; j < historyItems.Length; j++)
                            {
                                string fileAction = historyItems[j].GetFileAction().ToString();
                                uint version = historyItems[j].GetVersion();
                                AddOptions addOps = new AddOptions();
                                CheckoutOptions coOps = new CheckoutOptions();
                                coOps.SetExclusive(true);
                                CheckinOptions ciOps = new CheckinOptions();


                                switch (fileAction)
                                {

                                    case "AddToRepository":
                                        SSCMResult result = SSCMAPI.GetFile(context, oldRepo, scmfiles[i].GetName(), "C:\\migrate", TimestampType.CurrentDate, ref version, false);
                                        if (result.IsOk())
                                        {
                                            SetContext(newMainline, newMainline);

                                            SSCMAPI.AddFile(context, newRepo, workDir + "\\" + scmfiles[i].GetName(), "Added via API", addOps, 0);
                                            SetContext(oldBranch, oldMainline);
                                        }
                                        else
                                        {
                                            Logger.WriteLine("Error getting files from" + oldRepo + ".\n");
                                            Logger.WriteLine("Error reported: \n");
                                            Logger.WriteLine(SSCMAPI.GetLastError(result));
                                        }
                                        break;
                                    case "AddFromBranch":
                                        result = SSCMAPI.GetFile(context, oldRepo, scmfiles[i].GetName(), "C:\\migrate", TimestampType.CurrentDate, ref version, false);
                                        if (result.IsOk())
                                        {
                                            SetContext(newMainline, newMainline);

                                            SSCMAPI.AddFile(context, newRepo, workDir + "\\" + scmfiles[i].GetName(), "Added via API", addOps, 0);
                                            SetContext(oldBranch, oldMainline);
                                        }
                                        else
                                        {
                                            Logger.WriteLine("Error getting files from" + oldRepo + ".\n");
                                            Logger.WriteLine("Error reported: \n");
                                            Logger.WriteLine(SSCMAPI.GetLastError(result));
                                        }
                                        break;
                                    case "CheckIn":
                                        SetContext(newMainline, newMainline);
                                        result = SSCMAPI.Checkout(context, newRepo, scmfiles[i].GetName(), "C:\\migrate", "API CHECKOUT", coOps, 0);
                                        if (result.IsOk())
                                        {
                                            SetContext(oldBranch, oldMainline);
                                            result = SSCMAPI.GetFile(context, oldRepo, scmfiles[i].GetName(), "C:\\migrate", TimestampType.CurrentDate, ref version, false);
                                            if (result.IsOk())
                                            {
                                                SetContext(newMainline, newMainline);
                                                result = SSCMAPI.Checkin(context, newRepo, workDir + "\\" + scmfiles[i].GetName(), "Check in by API\n" + historyItems[j].GetComment(), ciOps, 0);
                                                if (!(result.IsOk()))
                                                {
                                                    Logger.WriteLine("Error checking in files to " + newRepo + ".\n");
                                                    Logger.WriteLine("Error reported: \n");
                                                    Logger.WriteLine(SSCMAPI.GetLastError(result));
                                                }
                                            }
                                            else
                                            {
                                                Logger.WriteLine("Error getting files from" + oldRepo + ".\n");
                                                Logger.WriteLine("Error reported: \n");
                                                Logger.WriteLine(SSCMAPI.GetLastError(result));
                                            }

                                        }
                                        else
                                        {
                                            Logger.WriteLine("Error checking out files from" + newRepo + ".\n");
                                            Logger.WriteLine("Error reported: \n");
                                            Logger.WriteLine(SSCMAPI.GetLastError(result));
                                        }
                                        SetContext(oldBranch, oldMainline);
                                        break;
                                    case "PromoteFromBranchWithMerge":
                                        SetContext(newMainline, newMainline);
                                        result = SSCMAPI.Checkout(context, newRepo, scmfiles[i].GetName(), "C:\\migrate", "API CHECKOUT", coOps, 0);
                                        if (result.IsOk())
                                        {
                                            SetContext(oldBranch, oldMainline);
                                            result = SSCMAPI.GetFile(context, oldRepo, scmfiles[i].GetName(), "C:\\migrate", TimestampType.CurrentDate, ref version, false);
                                            if (result.IsOk())
                                            {
                                                SetContext(newMainline, newMainline);
                                                result = SSCMAPI.Checkin(context, newRepo, workDir + "\\" + scmfiles[i].GetName(), "Check in by API\n" + historyItems[j].GetComment(), ciOps, 0);
                                                if (!(result.IsOk()))
                                                {
                                                    Logger.WriteLine("Error checking in files to " + newRepo + ".\n");
                                                    Logger.WriteLine("Error reported: \n");
                                                    Logger.WriteLine(SSCMAPI.GetLastError(result));
                                                }
                                            }
                                            else
                                            {
                                                Logger.WriteLine("Error getting files from" + oldRepo + ".\n");
                                                Logger.WriteLine("Error reported: \n");
                                                Logger.WriteLine(SSCMAPI.GetLastError(result));
                                            }

                                        }
                                        else
                                        {
                                            Logger.WriteLine("Error checking out files from" + newRepo + ".\n");
                                            Logger.WriteLine("Error reported: \n");
                                            Logger.WriteLine(SSCMAPI.GetLastError(result));
                                        }
                                        SetContext(oldBranch, oldMainline);
                                        break;
                                    case "PromoteFromBranchWithoutMerge":
                                        SetContext(newMainline, newMainline);
                                        result = SSCMAPI.Checkout(context, newRepo, scmfiles[i].GetName(), "C:\\migrate", "API CHECKOUT", coOps, 0);
                                        if (result.IsOk())
                                        {
                                            SetContext(oldBranch, oldMainline);
                                            result = SSCMAPI.GetFile(context, oldRepo, scmfiles[i].GetName(), "C:\\migrate", TimestampType.CurrentDate, ref version, false);
                                            if (result.IsOk())
                                            {
                                                SetContext(newMainline, newMainline);
                                                result = SSCMAPI.Checkin(context, newRepo, workDir + "\\" + scmfiles[i].GetName(), "Check in by API\n" + historyItems[j].GetComment(), ciOps, 0);
                                                if (!(result.IsOk()))
                                                {
                                                    Logger.WriteLine("Error checking in files to " + newRepo + ".\n");
                                                    Logger.WriteLine("Error reported: \n");
                                                    Logger.WriteLine(SSCMAPI.GetLastError(result));
                                                }
                                            }
                                            else
                                            {
                                                Logger.WriteLine("Error getting files from" + oldRepo + ".\n");
                                                Logger.WriteLine("Error reported: \n");
                                                Logger.WriteLine(SSCMAPI.GetLastError(result));
                                            }

                                        }
                                        else
                                        {
                                            Logger.WriteLine("Error checking out files from" + newRepo + ".\n");
                                            Logger.WriteLine("Error reported: \n");
                                            Logger.WriteLine(SSCMAPI.GetLastError(result));
                                        }
                                        SetContext(oldBranch, oldMainline);
                                        break;

                                }

                            }
                        }
                        else
                        {
                            Logger.WriteLine("Unable to retrieve historic information from files.");
                            Logger.WriteLine("Error reported: \n");
                            Logger.WriteLine(SSCMAPI.GetLastError(historyResult));

                        }
                    }
                }
            }
        }
    }
}
